/*
 *  main.cpp
 *
 *  Copyright (C) 2003 Mirko Boehm (mirko@kde.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#include <QLabel>
#include <QLayout>
#include <QCheckBox>
#include <QTimer>
#include <qslider.h>
//Added by qt3to4:
#include <QVBoxLayout>
#include <QDBusInterface>
#include <QDBusReply>

#include <kgenericfactory.h>
#include <kaboutdata.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kconfiggroup.h>
#include <kled.h>

#include "kcmkvaio_general.h"

#include "main.h"
#include "main.moc"

typedef KGenericFactory<KVaioModule, QWidget> KVaioModuleFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_kvaio, KVaioModuleFactory("kcmkvaio"))

#define CONFIG_FILE "kmilodrc"


KVaioModule::KVaioModule(QWidget *parent, const QStringList &)
    : KCModule(KVaioModuleFactory::componentData(), parent)
{
    KAboutData *about =
        new KAboutData(I18N_NOOP("kcmkvaio"), 0,
                       ki18n("KDE Control Module for Sony "
                                 "Vaio Laptop Hardware"),
                       0, KLocalizedString(), KAboutData::License_GPL,
                       ki18n("(c) 2003 Mirko Boehm"));

    about->addAuthor(ki18n("Mirko Boehm"),
                     ki18n("Original author"),
                     "mirko@kde.org");
    setAboutData( about );

    QVBoxLayout *layout = new QVBoxLayout(this);
    mKVaioGeneral = new KCMKVaioGeneral(this);
    layout->addWidget( mKVaioGeneral );
    layout->addStretch();

    mDriver = new KVaioDriverInterface(this);
    mDriverAvailable = mDriver->connectToDriver(false);
    mTimer = new QTimer(this);
    mTimer->start(231);

    load();
    if (! mDriverAvailable)
    {
        mKVaioGeneral->tlOff->show();
        mKVaioGeneral->frameMain->setEnabled(false);
        setButtons(buttons() & ~Default);
    } else {
        mKVaioGeneral->tlOff->hide();
        // mKVaioGeneral->frameMain->setEnabled (true);
    }

    connect(mKVaioGeneral, SIGNAL(changed()),
            SLOT(changed()));
    connect(mTimer, SIGNAL(timeout()), SLOT(timeout()));
    connect(mDriver, SIGNAL (vaioEvent(int)), SLOT (vaioEvent(int) ) );
}

void KVaioModule::save()
{
    if (! mDriverAvailable) return;

    kDebug() << "KVaioModule: saving." ;
    KConfig config(CONFIG_FILE);

    KConfigGroup cg(&config, "KVaio");

    cg.writeEntry("Report_Unknown_Events",
                      mKVaioGeneral->cbReportUnknownEvents->isChecked());
    cg.writeEntry("PeriodicallyReportPowerStatus",
		      mKVaioGeneral->mCbPowerMsgs->isChecked() );
    cg.writeEntry("PowerStatusOnBackButton",
		      mKVaioGeneral->mCbBackButtonMsg->isChecked() );
    config.sync();

    QDBusInterface kmilod("org.kde.kded", "/modules/kmilod", "org.kde.kmilod");
    QDBusReply<void> reply = kmilod.call("reconfigure");
    if( !reply.isValid() )
         kDebug() << "KVaio::showTextMsg: "
                  << "there was an error using D-Bus on kmilod::reconfigure()." << endl;
}

void KVaioModule::load()
{
    kDebug() << "KVaioModule: loading." ;
    KConfig config(CONFIG_FILE);

    KConfigGroup cg(&config, "KVaio");
    mKVaioGeneral->cbReportUnknownEvents->setChecked
        (cg.readEntry("Report_Unknown_Events", false));
    mKVaioGeneral->mCbPowerMsgs->setChecked
	(cg.readEntry("PeriodicallyReportPowerStatus", false) );
    mKVaioGeneral->mCbBackButtonMsg->setChecked
	(cg.readEntry("PowerStatusOnBackButton", true) );
}

void KVaioModule::defaults()
{
    mKVaioGeneral->cbReportUnknownEvents->setChecked( true );
    mKVaioGeneral->mCbPowerMsgs->setChecked( true );
    mKVaioGeneral->mCbBackButtonMsg->setChecked( true );

}

void KVaioModule::timeout()
{
    bool bat1Avail = false, bat2Avail = false, acConnected = false;
    int bat1Remaining = 0, bat1Max = 0, bat2Remaining = 0, bat2Max = 0;

    if(mDriver->getBatteryStatus(bat1Avail, bat1Remaining, bat1Max,
                                 bat2Avail, bat2Remaining, bat2Max,
                                 acConnected) )
    {
    }
    int remaining = (int)(100.0*(bat1Remaining+bat2Remaining)
                          / (bat1Max+bat2Max));
    mKVaioGeneral->mKPBattery->setProgress(remaining);
    mKVaioGeneral->kledBat1->setState(bat1Avail ? KLed::On : KLed::Off);
    mKVaioGeneral->kledBat2->setState(bat2Avail ? KLed::On : KLed::Off);
    mKVaioGeneral->kledAC->setState(acConnected ? KLed::On : KLed::Off);
}

void KVaioModule::vaioEvent (int e)
{
    kDebug () << "KVaioModule::vaioEvent: event: " << e ;
}

